from typing import List, Optional

from fastapi import APIRouter, Depends, HTTPException, Query, status
from sqlalchemy.ext.asyncio import AsyncSession

from app.dependencies.db import get_db
from app.schemas.product import ProductCreate, ProductResponse, ProductUpdate
from app.services import product as product_service

router = APIRouter(prefix="/products")


@router.get(
    "",
    response_model=List[ProductResponse],
    summary="Get all products",
    description="Retrieve all products with optional filtering",
)
async def get_products(
    skip: int = 0,
    limit: int = 100,
    anime_title: Optional[str] = Query(None, description="Filter by anime title"),
    character_name: Optional[str] = Query(None, description="Filter by character name"),
    db: AsyncSession = Depends(get_db),
):
    """Get all products with optional filtering."""
    products = await product_service.get_products(
        db,
        skip=skip,
        limit=limit,
        anime_title=anime_title,
        character_name=character_name,
    )
    return products


@router.get(
    "/{product_id}", response_model=ProductResponse, summary="Get a product by ID"
)
async def get_product(product_id: int, db: AsyncSession = Depends(get_db)):
    """Get a product by ID."""
    db_product = await product_service.get_product(db, product_id)
    if db_product is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Product with ID {product_id} not found",
        )
    return db_product


@router.post(
    "",
    response_model=ProductResponse,
    status_code=status.HTTP_201_CREATED,
    summary="Create a new product",
)
async def create_product(product: ProductCreate, db: AsyncSession = Depends(get_db)):
    """Create a new product."""
    return await product_service.create_product(db, product)


@router.put("/{product_id}", response_model=ProductResponse, summary="Update a product")
async def update_product(
    product_id: int, product_update: ProductUpdate, db: AsyncSession = Depends(get_db)
):
    """Update an existing product."""
    db_product = await product_service.get_product(db, product_id)
    if db_product is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Product with ID {product_id} not found",
        )
    return await product_service.update_product(db, db_product, product_update)


@router.delete(
    "/{product_id}",
    status_code=status.HTTP_204_NO_CONTENT,
    response_model=None,
    summary="Delete a product",
)
async def delete_product(product_id: int, db: AsyncSession = Depends(get_db)):
    """Delete a product."""
    db_product = await product_service.get_product(db, product_id)
    if db_product is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Product with ID {product_id} not found",
        )
    await product_service.delete_product(db, db_product)
    return None
